function [] = sirFFTandMotionComp(newConfig,logFile)
% Parameters: newConfig, full name of the script that initializes all
% required variables if no argument is passed "sirProcessConfig.m" in the
% local directory is run by default

% This code reads the raw data, parses and processes the Shallow Ice Radar
% data and saves them in batches as mat files
% Processsing steps
%   - Open and parse raw sir files
%   - Remove mean and scale
%   - Low pass filter data (optional)
%   - do a rvp correction and along-track dc signal removal, (optional)
%   - correct data for aircraft motion using GPS, (optional)
%   - coherently average data, (optional)
%   - perform fft on sir beat signal
%   - incoherently average data (optional)
%   - save the processed data
%
%  Tej Dhakal 02/01/2016
%  dhakal@ldeo.columbia.edu

try
    fprintf('************** sirFFTandMotionComp.m ****************\n')
    
    %% Argument checking and log file generation
    % A log file (diary) is generated if one isn't active already
    if nargin < 1
        configScript = 'sirProcessConfig.m';
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile, 'file')
            warning('sirFFTandMotionComp:existingLogFile', 'A logfile %s already exists!\n, Please cleanup the logs from %s for future processing', logFile, pwd);
            [~, logFileNameOnly, logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly, datestr(now, '_yyyymmdd_hhMMss'), logFileExt];
        end
        diary(logFile);
        localDiary = true;
    elseif nargin < 2
        configScript = newConfig;
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile, 'file')
            warning('sirFFTandMotionComp:existingLogFile', 'A logfile %s already exists!\n, Please cleanup the logs from %s for future processing', logFile, pwd);
            [~, logFileNameOnly, logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly, datestr(now, '_yyyymmdd_hhMMss'), logFileExt];
        end
        diary(logFile);
        localDiary = true;
    else
        configScript = newConfig;
        localDiary = false;
    end
    
    % This is the first process in the chain, clear(configScript) will
    % force runcount in the processconfig script to reset
    clear(configScript);
    
    run(configScript);
    
    tic
    % This will be checked agains configuration in the output dir (if exists)
    configParamsCurr = configParams; %#ok<NODEF> Defined in SirProcessConfig.m
    
    fprintf('sirFFTandMotionComp: Starting SIR level1 Process..\n')
    
    %% Pre-allocate common variables used for all flights
    tmpfile = eval([dataType, '(zeros(maxLinesperFile * frameLen, 1))']);   % holds raw data read from SIR file
    dataprestack = zeros(fftdatalen, maxLinesperFile);                   % holds raw sir date in 2d-array
    excessdata = zeros(fftdatalen, numAvg);                              % carries over excess data frames from file to file
    
    echoTimeutc_coh = zeros(1, ceil(maxLinesperFile / numCohAvg));         % % holds echotime for coherently averged data
    echoTimeutc = zeros(1, ceil(maxLinesperFile / numAvg));                % holds echotime for raw data
    
    datastack = zeros(fftdatalen, ceil(maxLinesperFile / numCohAvg));      % holds coherently averaged data in 2d-array
    datastackfft = zeros(NFFTr, ceil(maxLinesperFile / numCohAvg));        % holds frequency spectrum image in 2d- array
    
    
    if motionComp
        if lowMem
            del_tau = zeros(1, ceil(maxLinesperFile / numCohAvg));         % holds elevation offsets from reference elev
        else
            del_tau = zeros(fftdatalen, ceil(maxLinesperFile / numCohAvg)); %#ok<*USENS> Defined in SirProcessConfig.m
            H_MC = zeros(fftdatalen, ceil(maxLinesperFile / numCohAvg));   % holds full 2d phase offsets for elevation changes
        end
    end
    sirPwrEcho = zeros(1, ceil(maxLinesperFile / numCohAvg));              % holds average power for each eacho
    
    if process_metadata
        metadataprestack = zeros(numMetadataSamples, maxLinesperFile);   %holds metadta at full rate
        excessmetadata = zeros(numMetadataSamples, numAvg);
        
        % metadata are moved into 3rd dim when data are averaged
        metadatatmp = zeros(numMetadataSamples, ceil(maxLinesperbatch / numAvg), numAvg);
    end
    
    % temporary arrays for holding fft'd data, echo-power and utc times
    sirfftouttmp = zeros(numel(InRange), ceil(maxLinesperBatch / numAvg));
    sirpwrouttmp = zeros(1, ceil(maxLinesperBatch / numAvg));
    utctmp = zeros(1, ceil(maxLinesperBatch / numAvg));
    
    
    % This is a logical index array for full sir file that has been read
    % into tmpfile. It costs some flops to allocate but once set, it is
    % efficient at extracting data leaving the metadata and framing bytes
    % behind
    dataind = repmat(logical([zeros(markerLen, 1);zeros(numMetadataSamples, 1);...
        zeros(sigRange(1) - (markerLen + numMetadataSamples + 1), 1);ones(fftdatalen, 1);...
        zeros(frameLen - sigRange(end), 1)]), maxLinesperFile, 1); %#ok<FNCOLND> sigRange is defined in sirProcessConfig.m
    metadataind = repmat(logical([zeros(markerLen, 1);ones(numMetadataSamples, 1);...
        zeros(sigRange(1) - (numMetadataSamples + markerLen + 1), 1);zeros(fftdatalen, 1);...
        zeros(frameLen - sigRange(end), 1)]), maxLinesperFile, 1); %#ok<FNCOLND> sigRange is defined in sirProcessConfig.m
    
    switch lower(fftwintype)
        case 'cheb'
            fftwin = chebwin(fftdatalen, fftwinparam);
        case 'kaiser'
            fftwin = kaiser(fftdatalen, fftwinparam);
        case 'hanning'
            fftwin = hann(fftdatalen);
        case 'none'
            fftwin = 1;
        otherwise
            fftwin = hann(fftdatalen);
            warning('sirFFTandMotionComp:invalidWin', ...
                'Window type %s not recognized - a hanning window will be used', fftwintype);
    end
    
    % if lowMem is not set, all operations are performed on full 2d matrix
    % for a SIR file
    if ~lowMem
        fftwinMat = repmat(fftwin, 1, maxLinesperFile);
        tsampMat_full = repmat(tsamp, 1, ceil(maxLinesperFile));
    end
    
    if rvp_corr
        if lowMem
            rvpCorr = exp(-1j * pi * fbins_rvp.^2 / (bw / tau));
        else
            rvpCorrMat = repmat(exp(-1j * pi * fbins_rvp.^2 / (bw / tau)), 1, maxLinesperFile);
        end
    end
    
    %% start flight by flight processing
    for ff = 1:numel(uniqueFlightNum)
        FlightNo = uniqueFlight{ff};  %#ok<*IDISVAR> uniqueFlight is defined in sirProcessConfig.m
        fftandMotionCompTime = now;
        
        % set product level
        prodLevel.name = 'level1';
        if motionComp
            prodLevel.desc{1} = 'fftMotionComp';
        else
            prodLevel.desc{1} = 'fft';
        end
        prodLevel.time{1} = datestr(fftandMotionCompTime);
        
        %% Generate input/output paths
        rawdatapath = [sensorRootPath, filesep, rawDir, filesep, FlightNo, filesep, rawDataDir, filesep];
        fftProcessedDataPath = [sensorRootPath, filesep, process1Dir, filesep, stackDir, filesep, FlightNo, filesep, dataDir, filesep];
        processLogPath = [sensorRootPath, filesep, process1Dir, filesep, stackDir, filesep, FlightNo, filesep, logDir, filesep];
        
        if exist(rawdatapath, 'dir') == 0
            warning('sirFFTandMotionComp:noRawDir', 'Raw data flight %s directory does not exists!', FlightNo);
            continue;
        end
        
        %% Check input directory and sort data files
        dFileStr = dir([rawdatapath, rawFileKey]);
        numDataFiles = length(dFileStr);
        if numDataFiles == 0
            error('sirFFTandMotionComp:NorawFilesInDir', 'Data Files Not Found!')
        end
        
        % Sort data files by timestamp:
        for ii = 1:numDataFiles
            fileNameFields = strsplit(dFileStr(ii).name, fileFieldSep);
            if numel(fileNameFields) ~= numRawFileFields
                warning('sirFFTandMotionComp:InvalidFileNameFormat', ['File %s does not conform to SIR file format specifications\n', ...
                    'Please make sure all SIR data files in %s are in format "%s"\n %s will not be processed'], dFileStr(ii).name, rawdatapath, rawFileNameFormat, FlightNo)
                break;
            else
                dFileStr(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField}, fileNameFields{timeField}], [rawFileDateFormat rawFileTimeFormat]));
            end
        end
        if(ii < numDataFiles) % file sorting did not complete - lets move on to the next flight.
            warning('sirFFTandMotionComp:sortingIncomplete', 'All files canno not be sorted - %s will be skipped', FlightNo);
            continue;
        end
        clear fileNameFields;
        
        [~, sx] = sort([dFileStr.utc]);
        dFileStr = dFileStr(sx);
        fTime = [dFileStr.utc];
        
        %% load the GPS file for this flight
        gpsInpath = [gpsDataPath, filesep, trajectoryDir, filesep, FlightNo, filesep, txtDir, filesep];
        gpsFileList = dir([gpsInpath, '*', gpsProd, '*.txt']);
        
        % Take care of the case where there are gps files in parts
        try
            if numel(gpsFileList) == 1
                gpsFile = [gpsInpath, gpsFileList(1).name];
            elseif numel(gpsFileList) > 1
                gpsFile = combineGPStxtFileParts(gpsInpath, gpsProd, overwriteGPSmatFile);
            else
                warning('sirFFTandMotionComp:NoGPSFiles', 'No %s files available for this flight\n It will not be processed', gpsProd);
                continue;
            end
        catch errMsg
            fprintf('%s\n', errMsg.message);
            fprintf('%s\n', errMsg.identifier);
            for ejj = 1:numel(errMsg.stack)
                fprintf('In Function %s,  Line %d\n', errMsg.stack(ejj).name, errMsg.stack(ejj).line)
            end
            warning('sirFFTandMotionComp:failedToGetGPSData', 'Failed to get GPS data for %s\n It will not be processed', gpsProd, FlightNo);
            continue;
        end
        
        gpsMatFile = convertGPStxt2mat(gpsFile, 25, inf, [], overwriteGPSmatFile);
        load(gpsMatFile, 'Lat', 'Lon', 'Hell', 'Pitch', 'Roll', 'Heading', 'GPSUTC');
        
        %% Check for existing output directories and data (if any)
        if exist(fftProcessedDataPath, 'dir') == 0
            mkdir(fftProcessedDataPath)
        else
            dataFlist = dir([fftProcessedDataPath, '*', FlightNo, '*.mat']);
            if numel(dataFlist) > 0
                fprintf('sirFFTandMotionComp: Found existing data files\n');
                pcFlist = dir([fftProcessedDataPath, '*', configFnameRoot, '*.mat']);
                if numel(pcFlist) > 0
                    [~, sx] = sort([pcFlist.datenum]);
                    pcFlist = pcFlist(sx);
                    fprintf('sirFFTandMotionComp: Checking existing configuration data...\n')
                    load([fftProcessedDataPath, pcFlist(end).name], 'configParams');
                    if isequal(configParamsCurr.fftandMoCo, configParams.fftandMoCo)
                        if ignoreConfigMatch && overwriteoutput
                            warning('sirFFTandMotionComp:FlightDirExists', ['Configuration match ignored\n', 'All existing files will be deleted!\n', ...
                                'If this is not intended,  press Ctrl + C to cancel'], FlightNo);
                            
                            fprintf('sirFFTandMotionComp: Files will be deleted in %d seconds', 9);
                            for ss = 9:-1:0
                                fprintf('\b\b\b\b\b\b\b\b\b%d seconds', ss);
                                pause(1);
                            end
                            fprintf('\nDeleted!\n')
                            delete([fftProcessedDataPath, '*', FlightNo, '*.mat']);
                            delete([fftProcessedDataPath, '*onfig*.mat']);
                        else
                            fprintf(['sirFFTandMotionComp: Current settings are identical to the one %s processed data used.\n', ...
                                'Existing %s data will not be re-processed.\nTo force processing, Remove configuration mat file from\n%s\n'], FlightNo, FlightNo, fftProcessedDataPath);
                        end
                    else    % Configurations don't match
                        if overwriteoutput == 1
                            warning('sirFFTandMotionComp:FlightDirExists', ['New configuration!,  all existing files will be deleted!\n', ...
                                'If this is not intended, press Ctrl + C to cancel'], FlightNo);
                            
                            fprintf('sirFFTandMotionComp: Files will be deleted in %d seconds', 9);
                            for ss = 9:-1:0
                                fprintf('\b\b\b\b\b\b\b\b\b%d seconds', ss);
                                pause(1);
                            end
                            fprintf('\nDeleted!\n')
                            delete([fftProcessedDataPath, '*', FlightNo, '*.mat']);
                            delete([fftProcessedDataPath, '*onfig*.mat']);
                        else
                            warning('sirFFTandMotionComp:FlightDirExists', 'Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing', uniqueFlight{ff});
                            continue;
                        end
                    end
                else    %if for any reason there is no configuration file
                    if overwriteoutput == 1
                        warning('sirFFTandMotionComp:FlightDirExists', ['All existing files will be deleted!\n', ...
                            'If this is a mistake, press Ctrl + C to cancel'], FlightNo);
                        
                        fprintf('sirFFTandMotionComp: Files will be deleted in %d seconds', 9);
                        for ss = 9:-1:0
                            fprintf('\b\b\b\b\b\b\b\b\b%d seconds', ss);
                            pause(1);
                        end
                        fprintf('\nDeleted!\n')
                        delete([fftProcessedDataPath, '*', FlightNo, '*.mat']);
                        delete([fftProcessedDataPath, '*onfig*.mat']);
                    else
                        warning('sirFFTandMotionComp:FlightDirExists', 'Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing', uniqueFlight{ff});
                        continue;
                    end
                end
            end
        end
        % save a copy of configuration setting in output direcory for future
        % reference
        [~, configFilename, ~] = fileparts(configScript);
        copyfile([configFilename, '.mat'], [fftProcessedDataPath, configFilename, '.mat'])
        fprintf('sirFFTandMotionComp: Processing Flight %s\n', FlightNo)
        
        %% Now lets figure out if there are any existing processed files in the output directory.
        % If there are, processing will resume from there. If any averaging is
        % used, there may be gaps as long as numAvg number of echoes where the
        % processing software picks up.
        
        if exist('numExcessFrames', 'var')
            clear('numExcessFrames')            % Clear for each flight
        end
        level1Files = dir([fftProcessedDataPath, ['*', FlightNo, '*.mat']]);
        numProcessedfiles = numel(level1Files);
        if(numProcessedfiles > 1)
            for ii = 1:numProcessedfiles
                fileNameFields = strsplit(level1Files(ii).name, fileFieldSep);
                level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField}, fileNameFields{timeField}], [outFileDateFormat, outFileTimeFormat]));
            end
            [~, sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
            
            % Looking for a corrupt file, if found processing will start
            % there
            zeroSizeFile = find([level1Files.bytes] < 200, 1, 'first');
            if ~isempty(zeroSizeFile)
                numProcessedfiles = zeroSizeFile - 1;
                warning('sirFFTandMotionComp:corruptDataFile', 'Found a corrupt file %s, All files from this point on will be reprocessed', ...
                    level1Files(zeroSizeFile).name)
            end
            
            % Pick-up processing, check excess frames in the second last
            % file
            if numProcessedfiles > 1
                load([fftProcessedDataPath, level1Files(numProcessedfiles - 1).name], 'numexcessFrames', 'echotimeutc'); %#ok<*NASGU> Matlab is wrong! numexcessFrames is used if available
                lastProcessedEchoTime = echotimeutc(end);
            end
        end
        
        %% Figure out the files to process
        
        % Pick-up processing, if existing processed data use last prossed
        % echo as start time.
        if exist('lastProcessedEchoTime', 'var')
            startTime = datestr(unix2matlabtime(lastProcessedEchoTime), dateTimeFormat);
        end
        
        % startTime is kept as string. this allows users to specify a
        % period to process in processConfig file
        if ~exist('startTime', 'var')
            startTime = datestr(unix2matlabtime(fTime(1)), dateTimeFormat);
        end
        if ~exist('endTime', 'var')
            endTime = datestr(unix2matlabtime(fTime(end)), dateTimeFormat);
        end
        
        startutc = matlab2unixtime(datenum(startTime, dateTimeFormat));
        sfile = find([dFileStr.utc] < startutc, 1, 'last');
        endutc = matlab2unixtime(datenum(endTime, dateTimeFormat));
        efile = find([dFileStr.utc] > endutc, 1, 'first');
        clear startTime endTime startutc endutc;
        if isempty(sfile)
            startfile = 1;
        else
            startfile = sfile;
        end
        if isempty(efile)
            endfile = numDataFiles;
        else
            endfile = efile;
        end
        if startfile > 1
            if(endfile - startfile) < 2
                warning('sirFFTandMotionComp:InvalidStartEndTime', 'No files selected for processing! check the raw data directory')
                continue;
            end
        else
            if(endfile - startfile) < 1
                warning('sirFFTandMotionComp:InvalidStartEndTime', 'No files selected for processing! check the raw data directory')
                continue;
            end
        end
        
        %% Compute number of batches and files for each batch
        
        % Find truncated or missing files
        truncatedFile = [dFileStr.bytes] < frameLen * numAvg;
        missingFile = [false diff([dFileStr.utc]) > 11];      % This determines if files are contiguous
        
        % Compute initial number of batches:
        minfNum = zeros(ceil((endfile - startfile + 1) / filesperbatch) + numel(find(missingFile)) + numel(find(truncatedFile)) + 1, 1);
        maxfNum = minfNum;
        if(startfile > 1) % if this is pick-up processing
            % Excess frame from the last processed will be loaded before
            % the loop so start with the following file
            minfNum(1) = startfile + 1;
        else
            minfNum(1) = 1;
        end
        
        % Set Max file num to min then iterate
        maxfNum(1) = minfNum(1);
        
        % New batch starts every filesperbatch files
        % if there is a missing file that is not the first file of the
        % batch a new batch is started
        % if there is a truncated file that is not the first file of the
        % batch a new batch is started
        batchnumber = 1;
        for filenumber = minfNum(1) + 1:endfile
            numbatches = batchnumber;
            if missingFile(filenumber)
                if(filenumber ~= minfNum(batchnumber))
                    maxfNum(batchnumber) = filenumber - 1;   % current batch should end on previous file
                    minfNum(batchnumber + 1) = filenumber;   % next batch should start at the current file
                    maxfNum(batchnumber + 1) = minfNum(batchnumber + 1);
                    batchnumber = batchnumber + 1;
                end
            elseif filenumber == (minfNum(batchnumber) + filesperbatch - 1)
                maxfNum(batchnumber) = filenumber;
                minfNum(batchnumber + 1) = min(filenumber + 1, endfile);
                maxfNum(batchnumber + 1) = minfNum(batchnumber + 1);
                batchnumber = batchnumber + 1;
            elseif truncatedFile(filenumber)
                if(filenumber ~= minfNum(batchnumber))
                    maxfNum(batchnumber) = filenumber;
                    minfNum(batchnumber + 1) = min(filenumber + 1, endfile);
                    maxfNum(batchnumber + 1) = minfNum(batchnumber + 1);
                    batchnumber = batchnumber + 1;
                end
            elseif filenumber == endfile
                maxfNum(batchnumber) = filenumber;
            end
        end
        minfNum = minfNum(1:numbatches);
        maxfNum = maxfNum(1:numbatches);
        clearvars batchnumber filenumber;
        
        %% Process Data:
        if ~exist('numExcessFrames', 'var') % no pick-up processing
            numExcessFrames = 0;
        end
        
        % If this is a pick-up processing and there are left over frames lets
        % read the file here
        if numExcessFrames > 0
            if ~truncatedFile(startfile) && ~missingFile(startfile)
                numFramesInFile = dFileStr(ii).bytes / frameSize;
                numFramesToStack = numFramesInFile - numExcessFrames(1);
                numStackedFrames = numFramesToStack / numAvg;
                fid1 = fopen([rawdatapath, dFileStr(startfile).name], 'r');
                cleaner = onCleanup(@()fclose(fid1));
                tmpfile(1:numFramesInFile * frameSize / 2) = fread(fid1, numFramesInFile * frameSize / 2, dataType);
                clear('cleaner');
                dataprestack(:, 1:numFramesInFile) = reshape(tmpfile(dataind(1:frameLen * numFramesInFile)), fftdatalen, numFramesInFile);
                excessdata(:, 1:numExcessFrames) = dataprestack(:, numAvg * numStackedFrames + 1:end);
                if process_metadata
                    metadataprestack(:, 1:numFramesInFile) = reshape(tmpfile(metadataind(1:frameLen * numFramesInFile)), numMetadataSamples, numFramesInFile);
                    excessmetadata(:, 1:numExcessFrames) = metadataprestack(1:numMetadataSamples, numAvg * numStackedFrames + 1:end);
                end
            else
                numExcesFrames = 0;
                clear('lastProcessedEchoTime');
            end
        end
        
        %% Calulate time for the last echo (pick-up processing) or
        if exist('lastProcessedEchoTime', 'var')
            % This is the time for first echo after the last processed echo - last echo being the time for averaged echo
            lastTime = lastProcessedEchoTime + tau * numHWStacks * (numCohAvg + 1) / 2 + tau * numHWStacks;
            clear lastProcessedEchoTime; % clear this for next flight
        else
            lastTime = fTime(minfNum(1));
        end
        
        %% Loop through batches:
        isfirstfile = true;
        for batch = 1:numbatches
            
            % Communicate with human:
            fprintf('sirFFTandMotionComp: Processing batch %d of %d\n', batch, numbatches)
            
            % Reset echo counter:
            numechos = 1;
            % Loop through files:
            for ii = minfNum(batch):maxfNum(batch)
                
                if truncatedFile(ii)
                    warning('sirFFTandMotionComp:TruncatedFile', 'Warning: Skipping Truncated File %s', dFileStr(ii).name)
                    if(ii < endfile)
                        lastTime = fTime(ii + 1); % reset the start time
                    end
                    numExcessFrames = 0;     % reset excess frame count
                    continue
                end
                
                % update the lastTime and numExcessFrames var if there is a missing file
                if missingFile(ii)
                    lastTime = fTime(ii);
                    numExcessFrames = 0;
                end
                
                %% Setup Counters:
                % numFramesInFile = number of frames in current file
                % numAllFrames = Frames in files plus the excess frames from previous file
                % numReadFrames = number of frames to read from current data
                % numRemainingFrames = number of frames to be read from current data into excess Frame
                % numCohStackedFrames = number of frames to process after coherent stacking
                % numStackedFrames = number of frames after coherent and fft stacking
                
                numFramesInFile = dFileStr(ii).bytes / frameSize;
                numFramesToStack = (numFramesInFile + numExcessFrames) - mod((numFramesInFile + numExcessFrames), numAvg);
                numRemainingFrames = numFramesInFile - (numFramesToStack - numExcessFrames);
                numCohStackedFrames = numFramesToStack / numCohAvg;
                numStackedFrames = numFramesToStack / numAvg;
                
                %% Open and Read SIR data file
                fid1 = fopen([rawdatapath, dFileStr(ii).name], 'r');
                cleaner = onCleanup(@()fclose(fid1));
                tmpfile(1:numFramesInFile * frameLen) = fread(fid1, numFramesInFile * frameLen, dataType, dataEndian);
                clear('cleaner');
                
                % Calculate time for each echo, coherently averaged echo
                % and the final echo
                echoTimeutc_full(1:numFramesToStack) = lastTime + (0:numFramesToStack - 1) * tau * numHWStacks;
                echoTimeutc_coh(1:numCohStackedFrames) = lastTime + tau * numHWStacks * (numCohAvg - 1) / 2 + (0:numFramesToStack / numCohAvg - 1) * tau * numHWStacks * numCohAvg;
                echoTimeutc(1:numStackedFrames) =   lastTime + tau * numHWStacks * (numAvg - 1) / 2 + (0:numFramesToStack / numAvg - 1) * tau * numHWStacks * numAvg;
                lastTime = lastTime + (numFramesToStack - 1) * tau * numHWStacks;     % this is the time for the last (full rate) echo.
                
                if isfirstfile
                    isfirstfile = false;
                    if motionComp
                        if ~exist('userRefAlt', 'var')   % use elevation of the first echo as ref
                            init_Alt = interp1(GPSUTC, Hell, echoTimeutc_coh(1));
                        else
                            init_Alt = userRefAlt / 3.28;
                        end
                    end
                end
                
                % Check for edge cases
                if numFramesInFile == 0
                    warning('sirFFTandMotionComp:ZeroFileSize', 'Skipping empty file %s', dFileStr(ii).name);
                    continue
                end
                
                if numFramesToStack == 0
                    warning('sirFFTandMotionComp:notEnoughFrames', 'Not enough frames in %s - Skipping', dFileStr(ii).name);
                    continue
                end
                
                % Communicate with human:
                fprintf('sirFFTandMotionComp: Processing file %d of %d\n', ii - minfNum + 1, maxfNum - minfNum + 1)
                
                %% Read raw data and metadata into 2d arrays using logical maps
                dataprestack(:, 1:numExcessFrames + numFramesInFile) = [excessdata(:, 1:numExcessFrames), reshape(tmpfile(dataind(1:frameLen * numFramesInFile)), fftdatalen, numFramesInFile)];
                if(process_metadata == 1)
                    metadataprestack(:, 1:numExcessFrames + numFramesInFile) = [excessmetadata(:, 1:numExcessFrames), reshape(tmpfile(metadataind(1:frameLen * numFramesInFile)), numMetadataSamples, numFramesInFile + numExcessFrames)];
                end
                
                % Save excess data to be left after stacking
                if numRemainingFrames > 0
                    excessdata(:, 1:numRemainingFrames) = dataprestack(:, numFramesToStack + 1:numFramesInFile + numExcessFrames);
                    if(process_metadata == 1)
                        excessmetadata(:, 1:numRemainingFrames) = metadataprestack(1:numMetadataSamples, numFramesToStack + 1:numFramesInFile + numExcessFrames);
                    end
                end
                numExcessFrames = numRemainingFrames;
                
                %% remove dc offset from each trace and convert to analytical signal
                dataprestack(:, 1:numFramesToStack) = detrend(dataprestack(:, 1:numFramesToStack), 'constant') * elephantstoVolts;
                dataprestack(:, 1:numFramesToStack) = hilbert(dataprestack(:, 1:numFramesToStack));
                
                %% Remove along track dc signal
                if remove_meansig
                    dataprestack(:, 1:numFramesToStack) = fft(dataprestack(:, 1:numFramesToStack), [], 2);
                    dataprestack(:, [1, numFramesToStack]) = 0.0 + 0.0i;
                    dataprestack(:, 1:numFramesToStack) = ifft(dataprestack(:, 1:numFramesToStack), [], 2);
                end
                
                %% Apply residual video phase correction
                if rvp_corr
                    if lowMem
                        datastack(:, 1:numFramesToStack) = fftshift(fft(datastack(:, 1:numFramesToStack)), 1);
                        datastack(:, 1:numFramesToStack) = bsxfun(@times, datastack(:, 1:numFramesToStack), rvpCorr);
                        datastack(:, 1:numFramesToStack) = ifft(ifftshift(datastack(:, 1:numFramesToStack), 1));
                    else
                        datastack(:, 1:numFramesToStack) = ifft(ifftshift(fftshift(fft(datastack(:, 1:numFramesToStack)), 1) .* rvpCorrMat(:, 1:numFramesToStack), 1));
                    end
                end
                
                %% Compensate for aircraft motion
                if motionComp
                    if ~lowMem
                        del_tau(:, 1:numFramesToStack) = 2 * (repmat((interp1(GPSUTC, Hell, echoTimeutc_full(1:numFramesToStack)) - init_Alt), fftdatalen, 1)) / cAir;
                        H_MC(:, 1:numFramesToStack) = exp(-1j * 2 * pi * (f0 * del_tau(:, 1:numFramesToStack) + ...
                            Kr .*tsampMat_full(:, 1:numFramesToStack) .*del_tau(:, 1:numFramesToStack) - ...
                            1 / 2 * Kr * del_tau(:, 1:numFramesToStack).^2));
                        dataprestack(:, 1:numFramesToStack) = dataprestack(:, 1:numFramesToStack) .* H_MC(:, 1:numFramesToStack);
                    else
                        del_tau(1:numFramesToStack) = 2 * (interp1(GPSUTC, Hell, echoTimeutc_full(1:numFramesToStack)) - init_Alt) / cAir;
                        for ll = 1:fftdatalen
                            dataprestack(ll, 1:numFramesToStack) = dataprestack(ll, 1:numFramesToStack) .*exp(-1j * 2 * pi * (f0 * del_tau(1:numFramesToStack) + ...
                                Kr .*tsamp(ll) .*del_tau(1:numFramesToStack) - ...
                                1 / 2 * Kr * del_tau(1:numFramesToStack).^2));
                        end
                    end
                end
                
                %% Cohererent Averaging - Stacking
                % Stack data (mean)
                datastack(:, 1:numCohStackedFrames) = reshape(mean(reshape(dataprestack(:, 1:numFramesToStack), fftdatalen, numCohAvg, numCohStackedFrames), 2), fftdatalen, numCohStackedFrames);
                % Stack metadata:
                if(process_metadata == 1)
                    metadatatmp(:, numechos:numechos + numStackedFrames - 1, :) = permute(reshape(metadataprestack(:, 1:numFramesToStack), numMetadataSamples, numAvg, numStackedFrames), [1, 3, 2]);
                end
                
                %% Filter the data
                if usefilter
                    datastack(:, 1:numCohStackedFrames) = filtfilt(sirFilt, 1, datastack(:, 1:numCohStackedFrames));
                end
                
                %% Power Calculation for each trace
                sirPwrEcho(1, 1:numCohStackedFrames) = rms(datastack(:, 1:numCohStackedFrames));
                if normalizeEchoes
                    if ~lowMem
                        datastack(:, 1:numCohStackedFrames) = datastack(:, 1:numCohStackedFrames) ./  repmat(sirPwrEcho(1:numCohStackedFrames), fftdatalen, 1);
                    else
                        datastack(:, 1:numCohStackedFrames) = bsxfun(@rdivide, datastack(:, 1:numCohStackedFrames), sirPwrEcho(1:numCohStackedFrames));
                    end
                end
                
                %% FFT and incoherent averaging
                if lowMem
                    datastackfft(:, 1:numCohStackedFrames) = fft(bsxfun(@times, datastack(:, 1:numCohStackedFrames), fftwin), NFFTr);
                else
                    datastackfft(:, 1:numCohStackedFrames) = fft(fftwinMat(:, 1:numCohStackedFrames) .*datastack(:, 1:numCohStackedFrames), NFFTr);
                end
                if(numfftAvg > 1)
                    sirfftouttmp(:, numechos:numechos + numStackedFrames - 1) = reshape(mean(reshape(datastackfft(InRange, 1:numCohStackedFrames), numel(InRange), numfftAvg, numStackedFrames), 2), numel(InRange), numStackedFrames);
                    sirpwrouttmp(1, numechos:numechos + numStackedFrames - 1) = mean(reshape(sirPwrEcho(1:numCohStackedFrames), numfftAvg, numStackedFrames), 1);
                else
                    sirfftouttmp(:, numechos:numechos + numStackedFrames - 1) = datastackfft(InRange, 1:numStackedFrames);
                    sirpwrouttmp(1, numechos:numechos + numStackedFrames - 1) = sirPwrEcho(1:numStackedFrames);
                end
                utctmp(1, numechos:numechos + numStackedFrames - 1) = echoTimeutc(1:numStackedFrames);
                numechos = numechos + numStackedFrames;
            end
            
            %% Save the batch data
            if numechos < 2
                warning('sirFFTandMotionComp:noEchoesInFile', 'Too few echos for a batch : %d\n Batch will not be saved\n', numechos - 1)
                % Incremement file numbers:
                minfNum = minfNum + filesperbatch;
                maxfNu = min(maxfNum + filesperbatch, numDataFiles);
                continue
            end
            fprintf('sirFFTandMotionComp: Saving batch data\n');
            if(process_metadata == 1)
                metadata = metadatatmp(:, 1:numechos - 1, :);
            end
            
            % Create variables to save
            echotimeutc = utctmp(:, 1:numechos - 1);
            radecho = sirfftouttmp(:, 1:numechos - 1);
            sirPwrOut = sirpwrouttmp(:, 1:numechos - 1);
            
            filesinbatch = dFileStr(minfNum:maxfNum);
            numexcessFrames = numExcessFrames;
            
            txOn = sirPwrOut > txThreshold;
            
            lat = interp1(GPSUTC, Lat, echotimeutc);
            lon = interpDegWithJumps(GPSUTC, Lon, echotimeutc);
            hell = interp1(GPSUTC, Hell, echotimeutc);
            pitch = interpDegWithJumps(GPSUTC, Pitch, echotimeutc);
            roll = interpDegWithJumps(GPSUTC, Roll, echotimeutc);
            heading = interpDegWithJumps(GPSUTC, Heading, echotimeutc);
            
            %If data available, Create output batch name:
            if ~isempty(echotimeutc)
                % Create output batch name:
                outputname = [Campaign, fileFieldSep, FlightNo, fileFieldSep, ...
                    datestr(unix2matlabtime(echotimeutc(1)), [outFileDateFormat, ...
                    fileFieldSep, outFileTimeFormat]), fileFieldSep, prodLevel.name, fileFieldSep, Sensor, '.mat'];
                % Save output file:
                if(process_metadata == 1)
                    save([fftProcessedDataPath, outputname], 'Campaign', 'prodLevel', 'FlightNo', 'twtt', 'radecho', 'echotimeutc', ...
                        'fbins', 'fs', 'tau', 'NFFTr', 'tsamp', 'Range', 'InRange', 'metadata', 'filesinbatch', 'numCohAvg', 'numfftAvg', ...
                        'numexcessFrames', 'txOn', 'lat', 'lon', 'hell', 'pitch', 'roll', 'heading', 'sirPwrOut', 'fftandMotionCompTime');
                else
                    save([fftProcessedDataPath, outputname], 'Campaign', 'prodLevel', 'FlightNo', 'twtt', 'radecho', 'echotimeutc', ...
                        'fbins', 'fs', 'tau', 'NFFTr', 'tsamp', 'Range', 'InRange', 'filesinbatch', 'numCohAvg', 'numfftAvg', ...
                        'numexcessFrames', 'txOn', 'lat', 'lon', 'hell', 'pitch', 'roll', 'heading', 'sirPwrOut', 'fftandMotionCompTime');
                end
            end
        end
        
        %% Some housekeeping and cleanup
        warning('on', 'all')
        diary off;
        [~, logFileNameOnly, logFileExt] = fileparts(logFile);
        if ~exist(processLogPath, 'dir')
            mkdir(processLogPath);
        end
        movefile(logFile, [processLogPath, logFileNameOnly, fileFieldSep, 'fftandmotioncomp', datestr(now, '_yyyymmdd_hhMMss'), logFileExt]);
        [~, configFileNameOnly, ~] = fileparts(configScript);
        copyfile(configScript, [processLogPath, configFileNameOnly, datestr(fftandMotionCompTime, '_yyyymmdd_hhMMss'), '.m.log']);
        diary(logFile);
    end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0, 'Diary'), 'on')
            diary off;
            [~, ~, logext] = fileparts(logFile);
            if strcmpi(logext, '.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
    fprintf('sirFFTandMotionComp: Done!\n')
    toc
    
    %% Handle Errors
catch errMsg
    fprintf('sirFFTandMotionComp: Encountered an error\n');
    fprintf('%s\n', errMsg.message);
    fprintf('%s\n', errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n', errMsg.stack(jj).name, errMsg.stack(jj).line)
    end
    warning('on', 'all');
    % move the log file
    diary off;
    if exist(logFile, 'file') && exist('processLogPath', 'var')
        [~, logFileNameOnly, logFileExt] = fileparts(logFile);
        if ~exist(processLogPath, 'dir')
            mkdir(processLogPath);
        end
        movefile(logFile, [processLogPath, logFileNameOnly, datestr(now, '_yyyymmdd_hhMMss'), logFileExt]);
    end
    if debugEnable
        fprintf('sirFFTandMotionComp: Halting for debugging\n')
        keyboard
    end
end
end
