% This file gathers all parameters and file paths require for Shallow Ice
% Radar data in one script so that any function in the processing chain can
% call upon it to initialize all required variabled

% A copy of this script is saved in processing logs by the first function
% in the processing chain and a mat file with all parameters is saved in
% the processed data directory as well.
% Tej Dhakal 02/10/2016
% dhakal@ldeo.columbia.edu

%% This adds the commit hash to the version variable
version = '3.09.2';
try
    [~, commit_hash] = system('git rev - parse HEAD');
    version = [version, '.', commit_hash(1:8)];
catch
    warning('sirProcessConfig:InvalidPath', 'Cannot read commit hash - version number not updated');
end
%% Processing Settings: Change these as needed
if usejava('desktop')
    debugEnable = true;
else
    debugEnable = false;
end
SaveData = true;
motionComp = true;                 	% Uses GPS data to compensate for aircraft bulk motion deltaR
userRefAlt = 2500;                  % Altitude about which motion will be corrected
rvp_corr = false;
lowMem = true;                     	% Uses for loop for large matrices in a low memory system
remove_meansig = true;             	% Removes mean signal by taking along - track fft and setting dc to 0
numCohAvg = 4;                      % numbers of chirps to average before fft
numfftAvg = 1;                  	% numbers of chirps to average after fft
numAvg = numCohAvg * numfftAvg;       % total
normalizeEchoes = false;            % normalizes echoes by its average power
fftwintype = 'hanning';            	% 'cheb', 'kaiser', 'hanning'
fftwinparam = 5;                    %  100 for cheb, 5 - 15 for kaiser
process_metadata = false;                % parses and stores metadata info - disabling this might speed things a bit

% Use these (wisely) only if you need to process a small section of the flight
% starttime = '12 - 01 - 2014 09:38:00';    % UTC time of data to start processing at.  Format: 'dd - mm - yyyy HH:MM:SS'
% endtime = '12 - 01 - 2014 09:40:00';      % UTC time of data to end processing at.  Format: 'dd - mm - yyyy HH:MM:SS'
dateTimeFormat = 'yyyy / mm / dd HH:MM:SS';

usefilter = false;
filtertype = 'bandpass';                 % 'low' or 'bandpass' or 'none'
filtOrder = 8;

findLayers = true;                  % finds peaks and sorts them by power
numLayers = 2;                      % finds

txThreshold = 0.01;                 % vrms, Average power level to detect if transmission is on.
%% Project Campaign and data - These determine the input data location and ouotput product paths so type with care
rootpath = 'D:\trunk\';             % Root path for data repository

platform = 'ICEPOD';
platformDir = 'icepod';             % Root dir by project for level2 product and above
theaterDir = 'antarctica';          % Root dir by theatre for level1 product below
rawDir = 'raw';
CampaignYear = '20172018';          % This refers to campaign year name
Campaign = 'AN04';                  % Backward Compatibility
Project = 'RS03';                   % Make sure this is correct - will be used in processed file names
Sensor = 'SIR';                     % Sensor
DeploymentNum = 1;                  % Number of deployments this year

%% File level Directories
sensorDir = 'sir';
sensorTypeDir = 'radar';
logDir = 'log';
dataDir = 'mat';
matDir = 'mat';
txtDir = 'txt';
rawDataDir = 'bin';
decimatedDir = 'decimated';

%% Radar Settings:  These should not be changed
% Use different settings for different radar configurations
% AN01 Radar (Version 1) uses a 750 us chirp with 12.5 Mhz sampling frequency
% AN02 Radar (Version 2) uses a 250 us chirp with 50 Mhz sampling frequency

filePeriod = 10;
if strcmpi(Campaign, 'AN01')
    sirVersion = 1;
else
    sirVersion = 2;
end

switch sirVersion
    case 1
        fprintf('sirProcessConfig: Using settings for SIR hardware version: %d\n', sirVersion);
        fs = 12.5e6;            % Sampling Frequency
        tau = 750e-6;           % Chirp Period
        bw = 599996442;         % Chirp bandwidth Hz %599996441.603
        frameLen = 9352;        % number of samples collected for each chirp
        % data structure (samples): { < marker(2) >  (2) > < metadata(4) > < data(9352) > }{ < marker (2) > 
        sigRange = 1000:9000;   % discard first and last 2 samples (Marker Bytes)
        numHWStacks = 1;        % Number of hardware stacks
        dataType = 'uint16';      %
        numMetadataSamples = 0; % in number of samples; each sample is of dataType
        frameSize = frameLen * 2; % size of a frame in bytes
        rangeLim = [100 650];   % meters in Ice-distance surface ~ 427m - narrow range  =>  faster processing
        RangePass = rangeLim;   %m Used for bp filter by Range
    otherwise
        fprintf('sirProcessConfig: Using settings for SIR hardware version %d\n', sirVersion);
        fs = 50e6;              % Sampling Frequency
        tau = 250e-6;           % Chirp Period
        bw = 600138426;         % Chirp bandwidth Hz % 600138425.83
        frameLen = 12474;       % number of samples collected for each chirp
        % data structure (samples): { < marker(2) >  (2) > < metadata(4) > < data(12474) > }{ < marker (2) > 
        sigRange = 750:12000;   % discard samples at begining and end of the chirp
        numHWStacks = 16;       % Number of hardware stacks
        dataType = 'uint32';      %
        numMetadataSamples = 4; % starttime(unixsec) ppscount(unixsec) clkcount(sampleclk) echocount; in number of samples
        frameSize = frameLen * 4; % size of a frame in bytes;
        rangeLim = [100 874];   % meters in Ice-distance surface ~ 427m - narrow range  =>  faster processing
        RangePass = rangeLim;   %m by Range
end

prfPreSar = 1 / (tau * numHWStacks * numAvg); % chirp repition frequency
dataEndian = 'b';                       % Endianness of data being read
filesperbatch = 6;                      % Number of files compiled into each batch (integer)
maxLinesperFile = ceil(filePeriod / tau / numHWStacks * 1.1) + numAvg;     % max size for preallocation
maxLinesperBatch = ceil(filePeriod / tau / numHWStacks * 1.1) * filesperbatch + numAvg;
markerLen = 2;                              % lenth of data markers in samplecount 0xFFFF or 0xFFFFFFFF
f0 = 1.7e9;                                 % starting frequency
Kr = bw / tau;                                % Modulation rate
tsamp_full = linspace(0, tau - 1 / fs, fs * tau)';  % sample time within a chirp (fast time)
tsamp = tsamp_full(sigRange);               % sampling time for range of samples selected
fftdatalen = numel(sigRange);               % length of pre-fft chirp data
maxFFTLen = 32768;                          % determines the frequency resolution (more bins  =>  less power per bin)
elephantstoVolts = 2.415 / 2^16;              % adc conversion into volts

%% Compute Range and Frequency bins
NFFTr = max(2^nextpow2(fftdatalen), max(bestFFTLen(fftdatalen), maxFFTLen));
fbins = fs / 2 * linspace(0, 1, NFFTr / 2 + 1)';
fbins_rvp = fs / 2 * linspace( - 1, 1, fftdatalen)';  % this carries positive part of the spectrum including 0
twtt = fbins * tau / bw;
RangeFull = Fb2Range(bw, tau, fbins);
InRange = find(RangeFull >= rangeLim(1) & RangeFull <= rangeLim(2));
fbins = fbins(InRange);
Range = RangeFull(InRange);

%% Design a band pass filter if selected
% this needs to in sos format to be used with 0 - phase digital filter function 'filtfilt'
fpass = fbins([find(Range >= RangePass(1), 1, 'first'), find(Range <= RangePass(2), 1, 'last')]);
if usefilter
    if strcmpi(filtertype, 'low') %#ok < UNRCH > It will be reached if usefilter is enabled
        [z, p, k] = butter(filtOrder, fpass(2) / (fs / 2), 'low');
    else
        [z, p, k] = butter(filtOrder, fpass / (fs / 2), 'bandpass');
    end
    sirFilt = zp2sos(z, p, k);
end

%%%%%%%%%%%%%%%%%%%% Constants %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cIce = 1.68e8;      % speed of light in ice m / s
cAir = 299792458; 	% speed of light in vacuum m / s
nAir = 1.00029;     % Refractive index of Air - for reference only
nIce = 1.78;        % Refractive index of Ice-for reference only

%% Path for support functions required for SIR processing
matlab_common_path = ' / local / data / hestia / software / GIT2 / common / Matlab';
if ~(exist(matlab_common_path, 'dir') == 7)
    warning('sirProcessConfig:InvalidPath', 'Invalid path for matlab common functions. Please update the path');
else
    addpath(matlab_common_path);
end

%% Generate data and projec paths here
sensorRootPath = [rootpath, filesep, platformDir, filesep, theaterDir, filesep, CampaignYear, filesep, sensorTypeDir, filesep, sensorDir, filesep];

%% External surface file (Future work)
% surfaceFile = 'C:\Users\Tej\Desktop\Icepod\DICE\Matlab\AN1 - F001 - 20141117 - SPAN - OLD - FORMAT.txt';
surfaceSource = 'None'; % Tag to identify external surface

%% Line file contains the fligh#, line# and times for all the flights in the season
lineFileName = 'ICEPOD_20172018_LINES.txt';   % This file is expected to be in level2 PNT dir (see lineFile below)

%% Select lines and channels to process
processLines = {'all'}; % List of Lines to process e.g. {'L650', 'L670', 'L630', 'L380'}

%% Flight Parameters
surveyElev = 2500;      % ft, this is average survey elevation in ft
surveySpeed = 100;      % m / s, this is nominal speed

%% Parameters for Surface Identification / tracking
surfaceFilterLen = 20;      % m, smoothens the surface by this window
surfaceAcqWin = 600;        % m, used for initial acquision of surface for each batch
surfaceTrackingWin = 200;   % m, used for tracking the surface from echo to echo
numSurfPicks = 2;           % surfaces / layers are picked and sorted by power level
surfThresh = - 40;           % Threshold amplitude value for the peak (dB) ignore any peaks below this.
noTxSurfInd = 1;            % This is the index for surface when there is no transmission
forceSurfPick = true;
surfacePickWindowType = 'adaptive'; % 'adaptive' or 'fixed'
% 'adaptive' window selects random echoes across the file and computes the
% appropriate window for each file. 'fixed' will use the surfacTrackingWin
% and surfaceAcqWin and will be faster (in some cases) but will not work if
% the surface elevation varies by > surfaceTrackingWin / 2 within the batch
% (1 minute typical)

% Smoothing filter window for the surface in pixels
surfFilterWin = ceil(surfaceFilterLen / (tau * numAvg * numHWStacks * surveySpeed));

%% Options for bed pick data output
bedPickFilePeriod = 5;              % minutes. files will be broken into
bpSubsampleOption = 'average';     % 'decimate' or 'average' are recognized for now

%% Data output
overwriteoutput = false;      % If this is set, the output directories will be wiped before writing new files
ignoreConfigMatch = false;    % This forces processing even if the configuration for already processed data is identical to current one

%% Figure Parameters (Future Work)


%% Create a structure with a set of parameters
% This will be compared from process to process with with already processed
% data if they exist to determine if re-processing is required
configParams.fftandMoCo = struct('numCohAvg', numCohAvg, ...
    'numfftAvg', numfftAvg, 'filesperbatch', filesperbatch, 'rangeLim', rangeLim, ...
    'fftwintype', fftwintype, 'fftwinparam', fftwinparam, ...
    'txThreshold', txThreshold, 'surveyElev', surveyElev, 'surveySpeed', surveySpeed, ...
    'elephantstoVolts', elephantstoVolts, 'motionComp', motionComp, 'rvp_corr', rvp_corr, ...
    'remove_meansig', remove_meansig, 'normalizeEchoes', normalizeEchoes, ...
    'process_metadata', process_metadata, 'usefilter', usefilter, 'sigRange', sigRange, ...
    'filtertype', filtertype, 'filtOrder', filtOrder, 'RangePass', RangePass);

configParams.pickSurf = struct('surveySpeed', surveySpeed, 'surfaceFilterLen', surfaceFilterLen, ...
    'numSurfPicks', numSurfPicks, 'surfThresh', surfThresh, 'noTxSurfInd', noTxSurfInd);

% forcing the configuration script name to contain * onfig
configFnameRoot = 'onfig';
if isempty(regexp(mfilename, configFnameRoot, 'once'))
    error('diceProcessConfig:invalidFileName', ['Filename for the configuraiton script should have "config" or "Config" somewhere\n'...
        'This will make things easier down the line'])
end

configmatch = false;        % Leave this at false-calling function will change this
%% Generate directory fields / names
stackDir = num2str(numAvg, 'stack%02d');
process1Dir = 'fft';       % Geolocated compressed data
process2Dir = 'level2';       % SAR processed data

% Line Directory
projectlinesDir = 'projectlines';
docDir = 'doc';

% GPS Directory
pntDir = 'pnt';                 % Directory class
gpsDir =  'pod - ins';           % sensor directory for GPS data
trajectoryDir = 'trajectory';   % Data type
gpsProd = 'SPAN_FINAL';         % product to use-this must match the file naming scheme
gpsFileNameFormat = 'AN02_F1006_20151117_SPAN_FINAL.txt'; % Assumes gps file has this format
gpsDataPath = [rootpath, filesep, platformDir, filesep, theaterDir, filesep, CampaignYear, filesep, pntDir, filesep, gpsDir, filesep];
overwriteGPSmatFile = false;    % this ignores existing mat files and overwrites them

% Read the line file and determine flights to process
forceLinesReprocess = true;     % This regenerates the mat from the line file csv / text
[~, lineFileNameOnly, ~] = fileparts(lineFileName);
lineFile = [rootpath, filesep, platformDir, filesep, theaterDir, filesep, CampaignYear, filesep, docDir, filesep, projectlinesDir, filesep, txtDir, filesep, lineFileName];
lineMatFile = convertLineCsv2mat(lineFile, [], [], [], forceLinesReprocess);
clear lineFile;
load(lineMatFile, 'FlightNum', 'LineNum', 'LineTime');

%% Find Flight Numbers to Process based on Lines selected
% some functions work on entire flight - this will generate a list of
% unique numbers for use with those
if numel(processLines) >= 1
    if strcmpi(processLines{1}, 'all')
        processLines = LineNum;
    end
else
    warning('diceProcessConfig:NoLinesToProcess', 'No Lines Selected for Processing');
end
if any(~ismember(processLines, LineNum))
    fprintf('sirProcessConfig: Invalid Line Name(s)\nPlease remove any imaginary lines from the processLines list\n');
    fprintf('sirProcessConfig: Imaginary Lines:\n');
    InvalidLines = processLines(~ismember(processLines, LineNum));
    for invLn = 1:numel(InvalidLines)
        fprintf('%s\n', InvalidLines{invLn});
    end
    error('diceProcessConfig:invalidLineNum', 'Invalid Lines');
end

lineInds = ismember(LineNum, processLines);
LineNum = {LineNum{lineInds}}';
LineTime = LineTime(lineInds, :);
FlightNum = FlightNum(lineInds);
Flight = cellstr(num2str(1000 * DeploymentNum + FlightNum, 'F%04d'));
uniqueFlightNum = unique(FlightNum);
uniqueFlight = cellstr(num2str(1000 * DeploymentNum + uniqueFlightNum, 'F%04d'));

%% Data file stuff
rawFileNameFormat = 'AN01_F1002_20141119_084121_0430.sir';
fileFieldSep = '_';
numRawFileFields = numel(strsplit(rawFileNameFormat, fileFieldSep));
CampaignField = 1;
ProjectField = 1;
FlightNumField = 2;
LineNumField = 2;
dateField = 3;
timeField = 4;
LevelField = 5;
SensorField = 6;
fileLenTime = 10;
rawfileExt = 'sir';
rawFileDateFormat = 'yyyymmdd';
rawFileTimeFormat = 'HHMMSS';
outFileDateFormat = 'yyyymmdd';
outFileTimeFormat = 'HHMMSS';
level1FileNameFormat = 'AN02_F1007_20151112_010003_level1_SIR.mat';
level1aFileNameFormat = 'AN02_F1007_20151112_010003_level1a_SIR.mat';
level2FileNameFormat = 'AN02_L0605_20151112_010003_level2_SIR.mat';

rawFileKey = [' * .', rawfileExt];

save_append_backup = false;     % save a backup copy of data during append - will slow things down if enabled
tmpExt = '.tmp';                % used for backup file

%% Surface and Bedpick derived parameters
bedPickDecimate = max(round(16 / numAvg), 1);      % decimation factor for bed pick data
% This is number of traces per file before decimation
bedPickTracesPerFile = bedPickFilePeriod * 60 * prfPreSar - mod(bedPickFilePeriod * 60 * prfPreSar, bedPickDecimate);

surfaceAcqIndWin = ceil(surfaceAcqWin / mean(diff(Range)));
surfaceTrackingIndWin = ceil(surfaceTrackingWin / mean(diff(Range)));

initSurfInd = find(Range <= (surveyElev / 3.28), 1, 'last');

surfFilter = ones(1, surfFilterWin) / surfFilterWin;

% default setup of surfPickOption = 'descend' and numSurfPicks = 2 will output
% surfIndRad output that has two rows. the first row is the indices with
% surface return with max power. Inorder to get the topmost surface, use
% min(surfIndRad)
surfPickOption = 'descend'; % No other options available for now. This picks and sorts by amplitude

%% Product Level initialization
prodLevel.name = '?';
prodLevel.desc = {'?'};     % just a place holder - will be updated in each processing script
prodLevel.time = {'?'};

%% Just one more thing - keep track of how many times the script has been called
% runCount resets everytime sirProcessConfig.m is changed
% runCount is used to keep a single copy of the config script in the log
% directory only the first function to call upon this config script will
% increment runcount.
callername = dbstack;
clear ans;
if numel(callername) > 1  % only updates the count if some other funciton is calling on diceProcessConfigru
    save([mfilename, '.mat']);    % save all variables in the workspace
    runCount = runtracker();
end
function count = runtracker()
persistent runNum;
if isempty(runNum)
    runNum = 1;
else
    runNum = runNum + 1;
end
count = runNum;
end