% Sir Data Fixer
% This function replaces the incomplete frames from files with random noise so that they can be
% processed using the logical indices

% Note: This code can not fix/replace a corrupt first frame. There is at
% least one known instance where a frame header (marker) was split bettin
% end of a file and begining of the first file. In that case, the first
% file is padded with a dummy frame the the end and the first frame of
% second file is removed.

function newFilesize = fixSirFile(inputfile)

if nargin<1
    if usejava('desktop')
    [file,path] = uigetfile('*.sir','Select a sir file to fix');
    inputfile = fullfile(path,file);
    else
        error('Requires full path to sir file as argument')
    end
end
[~,filenameonly,fileExt] = fileparts(inputfile);
if ~strcmpi(fileExt,'.sir')
    warning('fixSirFile:invalidFileType','Invalid file typs %s extension not recognized',fileExt)
    return
end
fileNameFields = strsplit(filenameonly,'_');
if ismember('AN01',fileNameFields) || ismember('AN1',fileNameFields)
    sirversion = 1;
    framelen = 9352;
    frameSize = framelen*2;
    dataType = 'uint16';
    headerInds = 1:2;
    metaDataInds = 2;
    headerData = uint16(2^16-1);
    meanVal = 2^15;
    maxDiff = 1000;
else
    sirversion = 2;
    framelen = 12474;
    frameSize = framelen*4;
    dataType = 'uint32';
    headerInds = 1:2;
    metaDataInds = 3:6;
    headerData = uint32(2^32-1);
    meanVal = 2^19;
    maxDiff = 3000;
end
%% Check the file size and header to figure out if it needs fixing
fileInfo = dir(inputfile);
numframesInFile = fileInfo.bytes/frameSize;
if numframesInFile < 1
    return;
end
if  mod(numframesInFile,1)==0
    % framesize seems to be okay let's check the header
    fid1 = fopen(inputfile,'r');
    checkFilecleaner = onCleanup(@()fclose(fid1));
    dataMat = fread(fid1,[framelen,numframesInFile],['*' dataType],'b');
    clear('checkFilecleaner');
    if any(any(dataMat(headerInds,:) ~= headerData))
        fixThisFile = true;
        fprintf('Found misaligned header - Fixing...\n%s\n',inputfile);
    elseif any(abs(mean(dataMat(metaDataInds(end)+1:end,:))-double(meanVal))>maxDiff)
        fprintf('Found jumps in mean - Fixing...\n%s\n',inputfile);
        fixThisFile = true;
    else
        fixThisFile = false;
    end
else
    % file has some incomplete frames - need fixing
    fixThisFile = true;
    fprintf('Found partial frames - Fixing...\n%s\n',inputfile);
end

%create a copy of the input file first
if fixThisFile
    file2fix = [inputfile,'-orig'];
    if ~exist(file2fix,'file')
        copyfile(inputfile,file2fix);
    else
        warning('a copy already exists');
        newFilesize = 0;
        return;
    end
    %% Processing Flags
    rdFile = fopen(file2fix,'r');
    rdcleaner = onCleanup(@()fclose(rdFile));
    [rdData,readcount] = (fread(rdFile,'*uint8','b'));
    % each frame starts with 0xFFFFFFFF
    ind1 = find(rdData == 255);
    if sirversion > 1
	ind1(readcount-ind1 <7) =[];    % last frame needs to be alteast 8 bytes
    ind2 = find(rdData(ind1+1) == 255 & rdData(ind1 + 2) == 255 ...
        & rdData(ind1 + 3) == 255 & rdData(ind1 + 4) == 255 ...
        & rdData(ind1 + 5) == 255 & rdData(ind1 + 6) == 255 ...
        & rdData(ind1 + 7) == 255);
    else
        ind1(readcount-ind1 <3) =[];    % last frame needs to be atleast 4 bytes
        ind2 = find(rdData(ind1+1) == 255 & rdData(ind1 + 2) == 255 ...
        & rdData(ind1 + 3) == 255);
    end
    ind = ind1(ind2);
    if isempty(ind)
        warning('No frames in the file\n')
        newFilesize = 0;
        return;
    end
    inddiff = diff(ind);
    
    % logical map for good indices
    ind_good = (inddiff == frameSize | inddiff == frameSize-1);
    % There are many cases of an additional 0xFF (these are false positives)
    ind_falsepos = logical([inddiff==1;0]);
    % if the last frame has enough bytes consider it good
    ind_good = [ind_good;(readcount-ind(end))+1 >= frameSize];
    % Find any frames that may contain missing headers 
    ind_missing_start = [inddiff>frameSize;(readcount-ind(end))+1 > frameSize];
    
    % open output file to write
    wrFile = fopen([inputfile],'w+');
    wrcleaner = onCleanup(@()fclose(wrFile));
    
    clear rdData ind1 ind2 ind3 ind4
    
    % for each good index copy the frame
    % and for each bad index, write a frame with pseudorandom noise
    for pnum = 1:numel(ind)
        if ind_good(pnum)
            fseek(rdFile,(ind(pnum)-1),'bof');
            fwrite(wrFile,fread(rdFile,framelen,['*',dataType],'b'),dataType,'b');
            if  ind_missing_start(pnum)
                fseek(rdFile,(ind(pnum)-1),'bof');
                fwrite(wrFile,fread(rdFile,framelen,['*',dataType],'b'),dataType,'b');
            end
        elseif ~ind_falsepos(pnum)  % write dummy noise data to maintain timing
            if  ind_missing_start(pnum)
                fseek(rdFile,(ind(pnum)-1),'bof');
                fwrite(wrFile,fread(rdFile,framelen,['*',dataType],'b'),dataType,'b');
            end
            fwrite(wrFile,[headerData,headerData,headerData/2+randi(10,[1,framelen-2],dataType)],['*',dataType],'b');
        end
    end
    clear('rdcleaner','wrcleaner')
    finfo = dir(inputfile);
    newFilesize = finfo.bytes;
    fprintf('Replaced %d bad frames\n',numel(ind)+ numel(find(ind_missing_start)) - numel(find(ind_good))-numel(find(ind_falsepos)));
end